from core.routes import (connector_callback,
                         connector_command,
                         connector_message,)
from project.views import *


# Mapping handlers and views for Telegram Bot commands.
COMMANDS: list = [
]


MESSAGES: list = [
]


CALLBACKS: list = [
]



from core.routes import (connector_callback,
                                      connector_command,
                                      connector_message, )
from project.views import *


# Mapping handlers and views for Telegram Bot commands.
COMMANDS: list = [
    connector_command(view=main, commands="start, main",),
    connector_command(view=signin, commands=["signin"],),
    connector_command(view=signout, commands=["signout"],),
    connector_command(view=settings, commands=["settings"], ),
]


MESSAGES: list = [
    connector_message(view=signin, allowed_pre_route="signin"),
    connector_message(view=signup, allowed_pre_route="signup"),
    connector_message(view=delete, allowed_pre_route="delete"),
    connector_message(view=upload_docs, content_types=["document"], allowed_pre_route="settings"),
    connector_message(view=upload_image, content_types=["photo"], allowed_pre_route="settings"),
    connector_message(view=upload_location, content_types=["location"], allowed_pre_route="settings"),
    connector_message(view=send_location, allowed_pre_route="settings"),
    connector_message(view=last, content_types=["text", "location", "document"]),
]


CALLBACKS: list = [
    connector_callback(view=main, callback_data="main"),
    connector_callback(view=delete, callback_data="delete", allowed_pre_route="main",),
    connector_callback(view=signin, callback_data="signin", allowed_pre_route="main",),
    connector_callback(view=signup, callback_data="signup", allowed_pre_route="main",),
    connector_callback(view=settings, callback_data="settings", allowed_pre_route=["main", "settings"]),
    connector_callback(view=accounts, callback_data="accounts", allowed_pre_route=["main", "accounts"],),
    connector_callback(view=test, callback_data=["test1"], allowed_pre_route="accounts",),
    connector_callback(view=upload_docs, callback_data="upload_docs", allowed_pre_route="settings"),
    connector_callback(view=upload_image, callback_data="upload_image", allowed_pre_route="settings"),
    connector_callback(view=send_docs, callback_data="send_docs", allowed_pre_route="settings"),
    connector_callback(view=send_image, callback_data="send_image", allowed_pre_route="settings"),
    connector_callback(view=upload_location, callback_data="upload_location", allowed_pre_route="settings"),
    connector_callback(view=send_location, callback_data="get_location", allowed_pre_route="settings"),
    connector_callback(view=signout, callback_data="signout",),
    connector_callback(view=last),
]


import asyncio
from importlib import import_module
from telebot.async_telebot import (logger,
                                   AsyncTeleBot,)
from config import *


# create a Telegram Bot instance
bot = AsyncTeleBot(token=TELEBOT_TOKEN,
                   parse_mode=PARSE_MODE,
                   offset=OFFSET,
                   exception_handler=EXCEPTION_HANDLER,
                   state_storage=STATE_STORAGE,
                   disable_web_page_preview=DISABLE_WEB_PAGE_PREVIEW,
                   disable_notification=DISABLE_NOTIFICATION,
                   protect_content=PROTECT_CONTENT,
                   allow_sending_without_reply=ALLOW_SENDING_WITHOUT_REPLY,
                   colorful_logs=COLORFUL_LOG,
                   validate_token=VALIDATE_TOKEN)


# set Log Level
logger.setLevel(level=LOG_LEVEL)


# main for set menu and polling.
async def execute() -> None:
    await bot.set_my_commands(commands=MENU_COMMANDS)
    await bot.polling()
    return None


# start main
if __name__ == "__main__":
    # import handlers in installed app.
    for app_name in INSTALLED_APPS:
        try:
            module = import_module(f"{app_name}.routes")
            names_to_import = [name for name in dir(module) if not name.startswith("_")]
            for name in names_to_import:
                globals()[name] = getattr(module, name)

        except (ImportError, AttributeError) as e:
            if isinstance(e, ImportError):
                logger.error(f"Improper App Name '{app_name}': {e}")

            else:
                logger.error(f"accessing attributes in module {app_name}: {e}")

            break

    # execute polling in asynchronous
    asyncio.run(execute())


from core.handlers.handlers import (ReceiverBasic,
                                                 ReceiverWithForceReply,
                                                 CLIENT_INFO)
from translation import translate


class SignInBasic(ReceiverWithForceReply):
    class Meta:
        fields = ["email", "password"]
        fields_text = {
            "email": "signin_email",
            "password": "signin_password",
        }
        fields_regex = {
            "email": "^.+@.+\\..+$",
        }
        fields_error_msg = {
            "email": "warn_email_format",
        }

    def __init__(self, types, **kwargs):
        super(SignInBasic, self).__init__(types, **kwargs)

    async def get_client_data(self) -> bool:
        if not CLIENT_INFO[self.chat_id].get("is_signin"):
            return await super().get_client_data()

        else:
            await self.bot.send_message(chat_id=self.chat_id,
                                        text=translate(domain="default_warnings",
                                                       key="warn_already_signin",
                                                       language_code=self.language))
            return True


class SignOut(ReceiverBasic):
    async def pre_process(self) -> bool:
        if not CLIENT_INFO[self.chat_id].get("is_signin"):
            self.bot_text = translate(domain="default_warnings",
                                      key="warn_already_signout",
                                      language_code=self.language)
            return False
        return True

    async def send_message(self) -> None:
        if await self.pre_process():
            await self.post_process()

        await super().send_message()
        return None

    async def post_process(self):
        pass


class SignUp(ReceiverWithForceReply):
    class Meta:
        fields = ["email", "password"]
        fields_text = {
            "email": "signup_email",
            "password": "signup_password",
        }
        fields_regex = {
            "email": ("^.*@.+\\..+$", "^.+@.+\\.com"),
            "password": (
                "[A-Z]+",
                "[a-z]+",
                "[0-9]+",
                "[!@#$%^&*()_+\\-=]+",
            )
        }
        fields_error_msg = {
            "email": "warn_email_format",
            "password": (
                "warn_password_no_upper",
                "warn_password_no_lower",
                "warn_password_no_digit",
                "warn_password_no_special",
            )
        }

    def __init__(self, types, **kwargs):
        super(SignUp, self).__init__(types, **kwargs)


class DeleteAccount(ReceiverWithForceReply):
    class Meta:
        fields = ["password"]
        fields_text = {
            "password": "delete_account"
        }

    def __init__(self, types, **kwargs):
        super(DeleteAccount, self).__init__(types, **kwargs)



# please write down your code below.
from core.handlers import Message
from execute import bot
from project.serializers import *


# test
# main
async def main(types) -> None:
    handler_sr = Main(types=types, route="main")
    if await handler_sr.send_message():
        handler_sr = MainAuthorized(types=types,
                                    bot_text = "Welcome, Please select the one button below.")

    else:
        handler_sr = MainUnauthorized(types=types,
                                      bot_text = "You have to 'signin' or 'signup' first.",)

    await handler_sr.get_client_data()
    return None


# signin
async def signin(types):
    handler_sr = Signin(types=types, route="signin", link_route="main")
    if await handler_sr.get_client_data():
        await main(types=types)
    return None


# signup
async def signup(types):
    handler_sr = Signup(types=types, route="signup", link_route="settings")
    if await handler_sr.get_client_data():
        await main(types=types)
    return None


# signout
async def signout(types):
    handler_sr = Signout(types=types, route="signout")
    await handler_sr.send_message()
    await main(types=types)
    return None


# delete
async def delete(types):
    handler_sr = DeleteAccount(types=types, route="delete", link_route="main")
    if await handler_sr.get_client_data():
        await main(types=types)
    return None


# account in signin main
async def accounts(types):
    handler_sr = Accounts(types=types, route="accounts")
    await handler_sr.get_client_data()
    return None


# settings
async def settings(types):
    handler_sr = Settings(types=types,
                          bot_text="[File Handlers Test]",
                          route="settings",
                          link_route="settings")
    await handler_sr.send_message()
    return None


# upload file
async def upload_docs(types):
    handler_sr = UploadDocs(types=types,
                            bot_text="* Please Upload Json File",
                            link_route="settings")
    if await handler_sr.get_uploaded_file():
        await settings(types=types)
    return None


# send file
async def send_docs(types):
    handler_sr = SendDocs(types=types,
                          filename="test1.json",
                          link_route="settings")
    await handler_sr.send_message()
    return None


# send image
async def send_image(types):
    handler_sr = SendImage(types=types,
                           filename="img1.png",
                           link_route="settings")
    await handler_sr.send_message()
    return None


# upload image
async def upload_image(types):
    handler_sr = UploadImage(types=types,
                             bot_text="* Please upload image file",
                             link_route="settings")
    if await handler_sr.get_uploaded_file():
        await settings(types=types)
    return None


# upload location
async def upload_location(types):
    handler_sr = ReceiveLocation(types=types,
                                 bot_text="* Please send your location.",
                                 link_route="settings")
    if await handler_sr.get_location():
        await settings(types=types)
    return None


# send location
async def send_location(types):
    handler_sr = SendLocation(types=types,
                              bot_text="* Enter the name of location or postal number.",
                              link_route="settings")
    await handler_sr.send_message()
    return None


# test
async def test(types):
    handler_sr = Test(types=types, link_route="accounts")
    await handler_sr.send_message()
    return None


async def last(types):
    print(CLIENT_INFO[types.from_user.id])
    print(types)
    text = types.text if isinstance(types, Message) else types.data
    await bot.send_message(chat_id=types.from_user.id,
                           text=text)
    return None


import requests
from urllib.parse import urlencode


# please write down your code below.



API_URL_ROOT: str = "http://127.0.0.1:8000/"


async def api_get_info(data:dict):
    url = f"{API_URL_ROOT}api/v1/accounts/info/"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    response = requests.post(url=url,
                             headers=headers,
                             json=data)

    return response


async def api_signin(data:dict):
    url = f"{API_URL_ROOT}api/v1/accounts/signin/"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    response = requests.post(url=url,
                             headers=headers,
                             json=data)

    return response


async def api_signout(data:dict):
    url = f"{API_URL_ROOT}api/v1/accounts/signout/"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    response = requests.post(url=url,
                             headers=headers,
                             json=data)

    return response


async def api_signup(data:dict):
    url = f"{API_URL_ROOT}api/v1/accounts/signup/"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    response = requests.post(url=url,
                             headers=headers,
                             json=data)

    return response


async def api_delete_account(data:dict):
    url = f"{API_URL_ROOT}api/v1/accounts/delete/"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    response = requests.delete(url=url,
                             headers=headers,
                             json=data)

    return response


async def api_get_image(data=None):
    url = f"{API_URL_ROOT}api/v1/accounts/image/"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    response = requests.get(url=url,
                            headers=headers,
                            json=data)
    return response


async def api_update_location(data:dict):
    url = f"{API_URL_ROOT}api/v1/accounts/upload_geo/"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    response = requests.post(url=url,
                             headers=headers,
                             json=data)

    return response


# Set the argument for asynchronous telebot instance.

from os import getenv
from telebot.types import BotCommand


### TELEGRAM BOT CONFIG ###
# Set the Telegram Bot API TOKEN for Telegram Bot Father
# User must set this value before executing main.py
# it is recommended to export the value for environment variables.
# - sh: export TELEBOT_TOKEN=YOUR_BOT_TOKEN
TELEBOT_TOKEN: str = getenv("TELEBOT_TOKEN")


# Set the parse mode for Telegram Bot.
# default is None.
# possible values: "HTML", "Markdown"
PARSE_MODE: str | None = None


# Set the offset for Telegram Bot.
# default is None.
# possible value: positive int
# process the requests of message after specific ID' message
OFFSET: int | None = None


# Set the exception for Telegram Bot.
# import custom exception handler and assign it to EXCEPTION_HANDLER.
# default is handlers.exception_handlers.custom_exception_handler
EXCEPTION_HANDLER = None


# Set the way to store and maintain the status of Telegram Bot.
# bot will read the last status of telegram bot from file, DB or memory during restart process.
# default is None and it will store the status information in memory.
# possible values: class in telebot.storage package
# - StateMemoryStorage, StateStorageBase(use class inheriting this class), StateRedisStorage
STATE_STORAGE = None


# Set the web_page_preview config.
# it will show a preview of a linked web page(protocol://host.domain) in message or not
# default is None(False)
DISABLE_WEB_PAGE_PREVIEW: bool | None = False


# Set the notification for bot message(alarm or vibration on your phone).
# default is None(False) - will give you alarm or vibration.
DISABLE_NOTIFICATION: bool | None = False


# Set the message protection
# preventing users from copying and forwarding sent bot message.
# default is None(False)
PROTECT_CONTENT: bool | None = True


# Set the bot's action about sending message to user.
# bot can only send standalone message after user's request if the value is set for False
# default is None(True)
ALLOW_SENDING_WITHOUT_REPLY: bool | None = False


# Set the colorful log config
# require 'coloredlogs' package in pip before using it.
# default is None(False)
COLORFUL_LOG: bool | None = True


# Set whether the bot validates its token or not.
# default is True
VALIDATE_TOKEN: bool | None = True


# Import handlers
INSTALLED_APPS: list | tuple = (
    'project',
)

### TELEGRAM BOT CONFIG END ###

### TELEGRAM BOT FRAMEWORK CONFIG ###
# Set the allowed chat_type.
# default is ["private"]
# possible values: "private", "group", "supergroup", "channel"
ALLOWED_CHAT_TYPE: list = ["private"]


# Set menu button for your bot.
# use BotCommand(command="YOUR_COMMAND", description="COMMAND_DESCRIPTION")
# command name should be lower cases.
# if you change MENU_COMMANDS, remove your bot and re-enter to apply changes.
MENU_COMMANDS: list = [
    BotCommand(command="main", description="Main"),
    BotCommand(command="signin", description="Sign In"),
    BotCommand(command="signout", description="Sign Out"),
    BotCommand(command="settings", description="Settings"),
]

# Set chatting mode for your bot.
# SECRET_MODE is a bool variable that decides whether the previous messages will be removed or not.
# if True, bot will remove all remained messages in chat room after user types messages or click InlineMarkupButton
SECRET_MODE: bool = True

### TELEGRAM BOT FRAMEWORK CONFIG END###

### LOGGING CONFIG ###
# Set the log level
# default is ERROR
# possible values: "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"
LOG_LEVEL = "DEBUG"


### LOGGING CONFIG END ###



import yaml


def translate(domain:str, key:str, language_code:str="en") -> str:
    """
    this function is charge of translate string, which is defined on yaml file in the same path, into another language_code.
    if you need customize, please create another yaml file on translation folder,
    and use this function to translate.

    :param domain: name or alternate path of yaml file.
    :param key: name of main key in yaml file.
    :param language_code: language that you want to translate to.
    :return: str
    """

    # read yaml file into dictionary.
    with open(f"translation/{domain.replace("_", "/")}.yml", mode="r", encoding="utf-8") as file:
        content = dict(yaml.safe_load(file))

    try:
        # return translated string
        return content.get(key.lower()).get(language_code)

    # if there is no key in yml file, return original string(key)
    except AttributeError:
        return key


# please import handlers classes in core.handlers
# and inherit one of them to your custom handler serializer classes.
from core.handlers.handlers import *
from core.routes import CLIENT_INFO
from project.apis import *


# please write down your code below.



# please import handlers classes in core.handlers
# and inherit one of them to your custom handler serializer classes.
from core.handlers.handlers import *
from core.handlers.accounts import *
from core.handlers.file.docs import ReceiverWithCSVFile, SenderWithDocs
from core.handlers.file.multimedia import ReceiverWithImage, SenderWithImage
from core.handlers.geo import ReceiverWithLocation, SendWithLocationName
from core.routes import CLIENT_INFO

# test
from project.apis import *


# please write down your code below.
class Main(ReceiverBasic):
    async def send_message(self):
        response = await api_get_info(data={"chat_id": self.chat_id})
        CLIENT_INFO[self.chat_id].update({"is_signin": response.json().get("info").get("is_signin"),
                                          "info": response.json().get("info"),
                                          "language": self.types.from_user.language_code})

        if CLIENT_INFO[self.chat_id].get("is_signin"):
            return True

        else:
            return False


class MainAuthorized(ReceiverWithInlineMarkup):
    class Meta:
        fields = ("Accounts", "Settings", "Delete Account", "Sign Out",)
        fields_callback = {
            "Accounts": "accounts",
            "Settings": "settings",
            "Delete Account": "delete",
            "Sign Out": "signout",
        }


class MainUnauthorized(ReceiverWithInlineMarkup):
    class Meta:
        fields = ("Sign In", "Sign Up")
        fields_callback = {
            "Sign In": "signin",
            "Sign Up": "signup",
        }


class Signin(SignInBasic):
    def __init__(self, types, **kwargs):
        self.Meta = SignInBasic.Meta
        super().__init__(types, **kwargs)

    async def post_process(self) -> None:
        self.client_data.update({"chat_id": self.chat_id})
        response = await api_signin(data=self.client_data)

        if response.status_code == 200:
            self.bot_text = "Success to Signin."
            CLIENT_INFO[self.chat_id].update({
                "is_signin": True,
                "info": response.json().get("info"),
            })

        else:
            self.bot_text = "Fail to Signin."

        return None


class Signout(SignOut):
    async def post_process(self) -> None:
        data = {"email": CLIENT_INFO[self.chat_id]["info"]["email"],
                "chat_id": str(self.chat_id)}
        response = await api_signout(data=data)

        if response.status_code == 200:
            self.bot_text = "Success to signout"
            CLIENT_INFO[self.chat_id].update({"info": {}, "data": {}, "is_signin": False})
        else:
            self.bot_text = "Fail to signout"

        return None


class Signup(SignUp):
    async def post_process(self) -> None:
        self.client_data.update({"chat_id": self.chat_id,
                                 "first_name": self.request_user.first_name,
                                 "last_name": self.request_user.last_name})
        response = await api_signup(data=self.client_data)
        if response.status_code == 200:
            self.bot_text = "Success to Sign up. please sign in."

        else:
            self.bot_text = f"[{response.status_code}] {response.json().get("errors")}"

        return None


class DeleteAccount(DeleteAccount):
    async def post_process(self) -> None:
        self.client_data.update({"email": CLIENT_INFO[self.chat_id].get("info").get("email"),
                                 "chat_id": str(self.chat_id)})
        response = await api_delete_account(data=self.client_data)
        if response.status_code == 200:
            self.bot_text = "Successfully Delete your account."

        else:
            self.bot_text = f"[{response.status_code}] {response.json().get("errors")}"

        return None


class Accounts(ReceiverWithInlineMarkup):
    class Meta:
        fields = ["test1", "test2", "Cancel"]
        fields_callback = {
            "Cancel": "main"
        }
        fields_url = {"test2": "https://github.com/luna-negra"}


class Settings(ReceiverWithInlineMarkup):
    class Meta:
        fields  = ["Upload docs",
                   "Send docs",
                   "Upload Image",
                   "Send Image",
                   "Upload Location",
                   "Get Location",
                   "Cancel"]
        fields_callback = {
            "Cancel": "main"
        }


class UploadDocs(ReceiverWithCSVFile):
    async def post_process(self):
        print("Success to upload file")
        return None


class UploadImage(ReceiverWithImage):
    async def post_process(self):
        print("Success to upload image file")
        return None


class SendDocs(SenderWithDocs):
    async def pre_process(self) -> None:
        response = await api_get_info(data=CLIENT_INFO[self.chat_id].get("info"))
        if response.status_code == 200:
            self.content = response.json()

        else:
            self.bot_text = "Fail to get Image content"

            return None


class SendImage(SenderWithImage):
    async def pre_process(self) -> None:
        response = await api_get_image()
        if response.status_code == 200:
            self.content = response.content

        else:
            self.bot_text = "Fail to get Image content"

        return None


class ReceiveLocation(ReceiverWithLocation):
    async def post_process(self):
        self.location.update({"chat_id": self.chat_id})
        response = await api_update_location(self.location)
        if response.status_code == 200:
            self.bot_text = "Success to update location."

        else:
            self.bot_text = f"[ERROR] {response.reason}"

        return None


class SendLocation(SendWithLocationName):
    pass


class Test(ResultShowingWithInlineMarkup):
    async def pre_process(self):
        self.bot_text = f"You clicked {self.client_response}.\nPlease click 'Continue' if you want to continue."
        return None